@@ -18,6 +18,7 @@ from typing import (
     cast,
 )
 
+from . import collector
 from .collector import Collector
 from .types import ParamType
 
@@ -40,9 +41,28 @@ class AutoName(Enum):
     #: Derives the argument name from the original Python identifier (default)
     DERIVED = 1
 
+    @staticmethod
+    def derive_long_flag_name(name: str) -> str:
+        if name.endswith("_command"):
+            name = name[:-8]
+        return "--" + name.replace("_", "-")
+
+    @staticmethod
+    def derive_env_var_name(name: str, prefix: Optional[str]) -> str:
+        if prefix is not None:
+            return prefix + "_" + name.upper()
+        else:
+            return name.upper()
+
+    @staticmethod
+    def derive_config_key_name(name: str) -> str:
+        return name.replace("_", "-")
+
 
 ArgName = Union[str, AutoName]
 
+A = TypeVar("A", bound="Arg")
+
 
 @dataclass(frozen=True)
 class Arg(abc.ABC):
@@ -72,7 +92,7 @@ class Arg(abc.ABC):
             res.append(self.long_flag_name)
         return res
 
-    def updated_dict_(self, name: str, help: str, env_prefix: Optional[str]) -> Mapping[str, Any]:
+    def update_dict_(self, name: str, help: str, env_prefix: Optional[str]) -> Mapping[str, Any]:
         """
         Returns updated values for this argument, used during :class:`.App` construction
 
@@ -84,13 +104,13 @@ class Arg(abc.ABC):
         Returns:
 
         """
-        res = {"name": name, "help": help}
+        res = {"help": help}
         if self.long_flag_name == AutoName.DERIVED:
-            res["long_flag_name"] = "--" + name.replace("_", "-")
+            res["long_flag_name"] = AutoName.derive_long_flag_name(name)
         return res
 
-    def updated(self, name: str, help: str, env_prefix: Optional[str]) -> Arg:
-        return dataclasses.replace(self, **self.updated_dict_(name, help, env_prefix))
+    def updated(self: A, name: str, help: str, env_prefix: Optional[str]) -> A:
+        return dataclasses.replace(self, **self.update_dict_(name, help, env_prefix))
 
     def argparse_argument_kwargs(self) -> Mapping[str, Any]:
         """
@@ -103,7 +123,20 @@ class Arg(abc.ABC):
 
 
 @dataclass(frozen=True)
-class Expander(Arg):
+class Cmd(Arg):
+    # Base class for command arguments
+    pass
+
+
+@dataclass(frozen=True)
+class HelpCmd(Cmd):
+    """
+    Command-line argument that displays a help message and exits
+    """
+
+
+@dataclass(frozen=True)
+class Expander(Cmd):
     """
     Command-line argument that expands into a flag/value pair
     """
@@ -117,11 +150,8 @@ class Expander(Arg):
         """
         return (self.new_flag, self.new_value)
 
-    def updated(self, name: str, help: str, env_prefix: Optional[str]) -> Expander:
-        return dataclasses.replace(self, **self.updated_dict_(name, help, env_prefix))
-
     @staticmethod
-    def cmd(
+    def make(
         new_flag: str,
         new_value: str,
         *,
@@ -190,7 +220,7 @@ class Param(Arg, Generic[T]):
     """
 
     #: Argument type, parser from string to value
-    param_type: ParamType[T] = ParamType.invalid()
+    param_type: ParamType[T] = ParamType.invalid()  # type: ignore
 
     #: Argument collector
     collector: Collector[T] = Collector.invalid()  # type: ignore
@@ -217,22 +247,12 @@ class Param(Arg, Generic[T]):
     #: (and an underscore).
     env_var_name: ArgName = AutoName.FORBIDDEN
 
-    def __call__(self, config: Config) -> T:
-        assert self.name is not None, "Needs a constructed App instance to read values from"
-        return cast(T, config.values[self.name])
-
-    def updated(self, name: str, help: str, env_prefix: Optional[str]) -> Param[T]:
-        return dataclasses.replace(self, **self.replacements(name, help, env_prefix))
-
-    def replacements(self, name: str, help: str, env_prefix: Optional[str]) -> Mapping[str, Any]:
-        r = dict(super().updated_dict_(name, help, env_prefix))
+    def update_dict_(self, name: str, help: str, env_prefix: Optional[str]) -> Mapping[str, Any]:
+        r = {"name": name, **super().update_dict_(name, help, env_prefix)}
         if self.config_key_name == AutoName.DERIVED:
-            r["config_key_name"] = name.replace("_", "-")
+            r["config_key_name"] = AutoName.derive_config_key_name(name)
         if self.env_var_name == AutoName.DERIVED and env_prefix is not None:
-            if env_prefix:
-                r["env_var_name"] = env_prefix + "_" + name.upper()
-            else:
-                r["env_var_name"] = name.upper()
+            r["env_var_name"] = AutoName.derive_env_var_name(name, env_prefix)
         return r
 
     def all_config_key_names(self) -> Sequence[str]:
@@ -259,9 +279,15 @@ class Param(Arg, Generic[T]):
         else:
             return []
 
+    def is_required(self) -> bool:
+        """
+        Returns whether the argument is required
+        """
+        return self.default_value is None and self.collector.arg_required()
+
     def argparse_argument_kwargs(self) -> Mapping[str, Any]:
         res = super().argparse_argument_kwargs()
-        if self.default_value is None and self.collector.arg_required():
+        if self.is_required():
             res = {**res, "required": True}
         return {
             **res,
@@ -337,7 +363,7 @@ class Param(Arg, Generic[T]):
         """
         return Param(
             param_type=param_type,
-            collector=Collector.append(),
+            collector=Collector.append(),  # type: ignore
             default_value=None,
             positional=positional,
             short_flag_name=short_flag_name,
